# Linux method of reading characters

# Modules
import tty
import sys
import string
import termios
from .nix_maps import _NIX_MAP
from .. import keys

# Blanks
_BLANKS = [27, 49, 50, 79, 91, 126]

# Alphanumerics
_ALPHAS = string.digits + string.ascii_letters + string.punctuation + " "

# Key reader
def grabkey():

    # Try reading
    try:
        ch = ord(sys.stdin.read(1))

    except Exception:
        return None

    # Return our character
    return ch


import sys
import termios
import tty


def readchar():
    fd = sys.stdin.fileno()
    old_settings = termios.tcgetattr(fd)
    try:
        tty.setraw(sys.stdin.fileno())
        ch = sys.stdin.read(1)

        if ch == "\x1b":
            ch = sys.stdin.read(2)
            if ch == "[D":
                return keys.LEFT

            elif ch == "[C":
                return keys.RIGHT

            elif ch == "[A":
                return keys.UP

            elif ch == "[B":
                return keys.DOWN

        elif ch == "\r":
            return keys.ENTER

        elif ch == "\x7f":
            return keys.BACKSPACE

        elif ch == "\x03":
            return keys.CTRL_C

    finally:
        termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
    return ch

# Main function
def readchar_(set_tty: bool = True):
    fd = sys.stdin.fileno()
    old_settings = termios.tcgetattr(fd)

    # Try reading
    ch_map = []
    try:
        if set_tty:
            tty.setraw(sys.stdin.fileno())

        while not ch_map or ch_map[-1] in _BLANKS:
            ch = grabkey()
            ch_map.append(ch)

            # This code randomly shows up
            if ch_map[-1] == 126:
                ch_map = ch_map[:-1]

    finally:

        # Restore old terminal settngs
        if set_tty:
            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)

    # Return our character
    ch = ch_map[-1]

    print(ch, ch in _NIX_MAP)
    if ch in _NIX_MAP:
        return _NIX_MAP[ch]

    alpha = chr(ch)
    if alpha in _ALPHAS:
        return alpha

    return ch
